/*
Copyright (C) September 1999, Ka-Ping Yee

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "dynim.h"

/* DynImBuffer ========================================================= */

/* -------------------------------------------------------- construction */
DynImBuffer* dynim_buffer_new(int capacity, int depth)
    {
    DynImBuffer* buffer = alloc(DynImBuffer);
    int chunk = 256;
    while (chunk < capacity) chunk <<= 1;
    buffer->depth = depth;
    buffer->size = 0;
    buffer->capacity = chunk;
    buffer->data = allocarray(byte, capacity);
    buffer->bitsleft = 0;
    return buffer;
    }

void dynim_buffer_delete(DynImBuffer* buffer)
    {
    toast(buffer->data);
    toast(buffer);
    }


/* ----------------------------------------------------------- appending */
void dynim_buffer_extend(DynImBuffer* buffer, int capacity)
    {
    if (capacity <= buffer->capacity) return;
    while (buffer->capacity < capacity) buffer->capacity <<= 1;
    buffer->data = (byte*) realloc(buffer->data, buffer->capacity);
    }

void dynim_buffer_append(DynImBuffer* buffer, byte* data, int size)
    {
    dynim_buffer_extend(buffer, buffer->size + size);
    memcpy(buffer->data + buffer->size, data, size);
    buffer->size += size;
    }

void dynim_buffer_putbits(DynImBuffer* buffer, long value, int bits)
    {
    byte* ptr;
    int bitsleft = buffer->bitsleft, i;

    dynim_buffer_extend(buffer, buffer->size + 8); 
    ptr = buffer->data + buffer->size - 1;

    while (bits)
        {
        *ptr |= (byte) (value << (8 - bitsleft));
        if (bits > bitsleft)
            {
            value >>= bitsleft;
            bits -= bitsleft;
            bitsleft = 8;
            buffer->size++;
            *(++ptr) = 0;
            }
        else
            {
            bitsleft -= bits;
            bits = 0;
            }
        }

    buffer->bitsleft = bitsleft;
    }
