/*
Copyright (C) September 1999, Ka-Ping Yee

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef DYNIM_H
#define DYNIM_H

#include <stdlib.h>
#include <stdio.h>

#define alloc(type) ((type*) malloc(sizeof(type)))
#define allocarray(type, size) ((type*) malloc(sizeof(type)*(size)))
#define toast(ptr) do { if (ptr) free(ptr); ptr = NULL; } while (0)

typedef unsigned char byte;
typedef unsigned int word;

typedef struct
    {
    byte r, g, b;
    }
DynImColour;

typedef struct
    {
    int size;
    DynImColour* colours;
    }
DynImPalette;

typedef struct
    {
    int width, height;
    DynImPalette* palette;
    byte* data;
    }
DynImCanvas;

typedef struct
    {
    int left, top, right, bottom;
    int origin, offset, baseline;
    }
DynImMetric;

typedef struct
    {
    int size;
    DynImMetric* metrics;
    int ascent, descent;
    DynImCanvas* canvas;
    }
DynImFont;

typedef struct
    {
    byte depth;
    int size, capacity;
    byte* data;
    int bitsleft;
    }
DynImBuffer;

DynImPalette* dynim_palette_new(int size);
void dynim_palette_delete(DynImPalette* palette);
DynImPalette* dynim_palette_clone(DynImPalette* source);

DynImCanvas* dynim_canvas_new(DynImPalette* palette, int width, int height);
void dynim_canvas_delete(DynImCanvas* canvas);

DynImCanvas* dynim_canvas_loadppm(int handle);
DynImCanvas* dynim_canvas_clone(DynImCanvas* source);
DynImCanvas* dynim_canvas_copy(DynImCanvas* source, byte fill,
                               int l, int t, int r, int b);
void dynim_canvas_paste(DynImCanvas* canvas, DynImCanvas* source, int x, int y);

void dynim_canvas_clear(DynImCanvas* canvas, byte colour);
void dynim_canvas_setpalette(DynImCanvas* canvas, DynImPalette* palette);
#define dynim_canvas_pixelptr(canvas, x, y) \
    (&((canvas)->data[((x) + (y)*((canvas)->width))]))
#define dynim_canvas_pixel(canvas, x, y) \
    (*dynim_canvas_pixelptr(canvas, x, y))

void dynim_canvas_drawrect(DynImCanvas* canvas, byte colour,
                           int a, int b, int x, int y);
void dynim_canvas_fillrect(DynImCanvas* canvas, byte colour,
                           int a, int b, int x, int y);
int dynim_canvas_drawtext(DynImCanvas* canvas, DynImFont* font,
                          char* text, int x, int y, int shift);

DynImFont* dynim_font_newfromcanvas(DynImCanvas* canvas);
DynImMetric* dynim_font_getmetric(DynImFont* font, char ch);
int dynim_font_textwidth(DynImFont* font, char* text);
void dynim_font_delete(DynImFont* font);

DynImBuffer* dynim_buffer_new(int capacity, int depth);
void dynim_buffer_delete(DynImBuffer* buffer);
void dynim_buffer_extend(DynImBuffer* buffer, int capacity);
void dynim_buffer_append(DynImBuffer* buffer, byte* data, int size);

void dynim_gif_start(int handle, int width, int height);
void dynim_gif_putappext(int handle, char* appid, char* appauth,
                         byte* data, int size);
void dynim_gif_putcomment(int handle, char* text);
void dynim_gif_putcanvas(int handle, DynImCanvas* canvas,
                         DynImBuffer* buffer, int x, int y);
void dynim_gif_end(int handle);
DynImBuffer* dynim_gif_encode(DynImCanvas* canvas);

#endif
