/*
Copyright (C) September 1999, Ka-Ping Yee

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "dynim.h"

/* DynImFont =========================================================== */

DynImMetric zerometric = {0, 0, 0, 0, 0, 0, 0};

/* -------------------------------------------------------- construction */
DynImFont* dynim_font_newfromcanvas(DynImCanvas* canvas)
    {
    DynImFont* font = alloc(DynImFont);
    byte ch = 0;
    int x, ix, bx, bl, br, ascent, descent;
    int y, iy, baseline, rowtop, rowbot;
    font->metrics = allocarray(DynImMetric, 256);

#define pix(x, y) dynim_canvas_pixel(canvas, x, y)

    font->ascent = -999;
    font->descent = -999;

    for (ch = 0; ch < 32; ch++)
        {
        font->metrics[ch] = zerometric;
        }
    ch = 32;

    y = 0;
    while (y < canvas->height)
        {
        /* find vertical row bounds */
        while (y < canvas->height && !pix(0, y)) y++;
        if (y >= canvas->height) break;
        rowtop = y;

        while (y < canvas->height && pix(0, y)) y++;
        rowbot = y;
        
        y = rowtop;
        while (y < rowbot && !pix(1, y)) y++;
        if (y >= rowbot) break; /* no baseline marker found */

        baseline = y;
        y = rowbot;
        bx = x = 2;

        while (x < canvas->width && bx < canvas->width)
            {
            /* find char origin and offset */
            while (bx < canvas->width && !pix(bx, rowtop)) bx++;
            if (bx >= canvas->width) break;
            bl = bx;

            while (bx < canvas->width && pix(bx, rowtop)) bx++;
            br = bx - 1;

            if (pix(br, rowtop+1))
                {
                font->metrics[ch].origin = bl;
                font->metrics[ch].offset = br - bl;
                }
            else if (pix(bl, rowtop+1))
                {
                font->metrics[ch].origin = br + 1;
                font->metrics[ch].offset = bl - br;
                }
            else break; /* no direction indicator */

            /* find ink left-right bounds */
            while (x < canvas->width && !pix(x, rowtop+2)) x++;
            if (x >= canvas->width) break;
            font->metrics[ch].left = x;

            while (x < canvas->width && pix(x, rowtop+2)) x++;
            font->metrics[ch].right = x;

            /* find ink top-bottom bounds */
            ix = font->metrics[ch].left - 1;
            while (ix >= 0)
                {
                iy = rowtop;
                while (iy < rowbot && !pix(ix, iy)) iy++;
                if (iy >= rowbot)
                    {
                    ix--;
                    continue;
                    }
                font->metrics[ch].top = iy;

                while (iy < rowbot && pix(ix, iy)) iy++;
                font->metrics[ch].bottom = iy;

                break;
                }

            font->metrics[ch].baseline = baseline;

            ascent = font->metrics[ch].baseline - font->metrics[ch].top;
            descent = font->metrics[ch].bottom - font->metrics[ch].baseline;
            if (ascent > font->ascent) font->ascent = ascent;
            if (descent > font->descent) font->descent = descent;

            ch++;
            }
        }

    font->canvas = dynim_canvas_clone(canvas);
    font->size = ch;
    return font;
    }

void dynim_font_delete(DynImFont* font)
    {
    toast(font->metrics);
    dynim_canvas_delete(font->canvas);
    toast(font);
    }

DynImMetric* dynim_font_getmetric(DynImFont* font, char ch)
    {
    unsigned char c = ch;
    if (c < font->size) return &(font->metrics[c]);
    else return &zerometric;
    }

int dynim_font_textwidth(DynImFont* font, char* text)
    {
    DynImMetric* metric;
    int width = 0;

    for (; *text; text++)
        {
        metric = dynim_font_getmetric(font, *text);
        width += metric->offset;
        }

    return width;
    }
