/*
Copyright (C) September 1999, Ka-Ping Yee

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "table.h"

Table* table_new(int nbuckets)
    {
    int i;
    Table* table = alloc(Table);
    table->nbuckets = nbuckets;
    table->buckets = allocarray(TableEntry*, nbuckets);
    for (i = 0; i < nbuckets; i++)
        table->buckets[i] = 0;
    return table;
    }

void table_delete(Table* table)
    {
    table_clear(table);
    toast(table->buckets);
    toast(table);
    }

void table_clear(Table* table)
    {
    int i;
    TableEntry* entry;
    TableEntry* nextentry;

    for (i = 0; i < table->nbuckets; i++)
        {
        entry = table->buckets[i];
        while (entry)
            {
            nextentry = entry->next;
            toast(entry->key);
            toast(entry);
            entry = nextentry;
            }
        table->buckets[i] = 0;
        }
    }

word table_hash(byte* key, int size)
    {
    int i;
    word hash = 0;

    for (i = 0; i < size; i++)
        hash = (1000003*hash) ^ key[i];
    return hash;
    }

void table_put(Table* table, byte* key, int size, int value)
    {
    TableEntry* entry = alloc(TableEntry);
    word hash = table_hash(key, size);
    int index = hash % table->nbuckets;

    entry->key = allocarray(byte, size);
    memcpy(entry->key, key, size);
    entry->size = size;
    entry->value = value;
    entry->hash = hash;

    entry->next = table->buckets[index];
    table->buckets[index] = entry;
    }

int table_get(Table* table, byte* key, int size)
    {
    word hash = table_hash(key, size);
    int index = hash % table->nbuckets;
    TableEntry* entry;
    
    for (entry = table->buckets[index]; entry; entry = entry->next)
        {
        if (entry->hash == hash && entry->size == size &&
            memcmp(entry->key, key, size) == 0)
            return entry->value;
        }

    return -1;
    }
